#include <stdio.h>
#include <ctype.h>
#include <math.h>

/* Stopword removal, remove word (-w) or whole line (default) */

#define MAXLEN		65536
#define STOPLIST	"/home/io/staff/jz/src/stop/stoplist"
#define true		1
#define false		0

/*    Structure to hold term */
typedef struct termrec
{
    char	*term;			/* indexed word */
    struct termrec *next;
} node;


int	casesens = true;			/* case sensitive */

main(int argc, char *argv[])
{
    int		line = true;
    char	*stoplist = STOPLIST;


restart:
    if( argc > 1 )
    {
	if( strcmp(argv[1], "-w") == 0 )
	{
	    line = false;
	    argc--;
	    argv++;
	    goto restart;
	}
	if( strcmp(argv[1], "-i") == 0 )
	{
	    casesens = false;
	    argc--;
	    argv++;
	    goto restart;
	}
	else
	{
	    stoplist = argv[1];
	}
	/* else
	{
	    fprintf(stderr, "Usage: rmstop [-w] [-i] [stopfile]\n");
	    exit(1);
	} */
    }


    init_table();
    load_stop(stoplist);

    if( line )
	rmline();
    else
	rmword();

    exit(0);
}


/* Load stop words into hash table */
load_stop(char *stoplist)
{
    FILE	*fp, *fopen();
    char	buf[MAXLEN], *w, *fgets(), *strchr();
    node	*insert();
    
    if( ( fp = fopen(stoplist, "r") ) == NULL )
    {
	fprintf(stderr, "Can't open stoplist %s.\n", stoplist);
	exit(1);
    }
    while( fgets(buf, MAXLEN, fp) != NULL )
    {
	*strchr(buf, '\n') = '\0';
	(void) insert(buf);
    }
    fclose(fp);
}


/* Process stdin, remove all lines with a stopword */
rmline()
{
    char	*w, *s, tmp[MAXLEN], line[MAXLEN], *gets();
    node	*find();

    while( ( w = gets(line) ) != NULL )
    {
	while( *w != '\0' )
	{
	    while( !isalnum(*w) && *w != '\0' )
		w++;
	    if( casesens )
		for( s=tmp ; isalnum(*w) ; w++, s++ )
		    *s =  *w;
	    else
		for( s=tmp ; isalnum(*w) ; w++, s++ )
		    *s = ( isupper(*w) ) ? tolower(*w) : *w;
	    *s = '\0';
	    if( find(tmp) != NULL )
		goto cont;
	}
	printf("%s\n", line);

cont:
	continue;
    }
}


/* Process stdin, remove all stopword occurrences */
rmword()
{
    char	*b, *s, *w, *p, tmp[MAXLEN], line[MAXLEN], buf[MAXLEN], *gets();
    node	*find();

    while( ( w = gets(line) ) != NULL )
    {
	buf[0] = '\0';
	b = buf;
	while( *w != '\0' )
	{
	    for( ; !isalnum(*w) && *w != '\0' ; w++, b++ )
		*b = *w;
	    p = b;
	    if( casesens )	/* this could be more efficient! */
		for( s=tmp ; isalnum(*w) ; w++, b++, s++ )
		{
		    *s = *w;
		    *b = *w;
		}
	    else
		for( s=tmp ; isalnum(*w) ; w++, b++, s++ )
		{
		    *s = ( isupper(*w) ) ? tolower(*w) : *w;
		    *b = *w;
		}
	    *s = '\0';
	    if( find(tmp) != NULL )
		b = p;
	}
	*b = '\0';
	printf("%s\n", buf);
    }
}


#define    TABLESIZE    2048
#define    NPRIMES	20

int    primes[] = { 101749, 190763, 165877, 190097, 84673,
		    193357, 157721, 102397, 103837, 197803,
		    190753, 200323, 101111, 111659, 222029,
		    14347, 92219, 23053, 77999, 220373 };

node    *table[TABLESIZE];

init_table()
{
    int    i;

    for( i=0 ; i<TABLESIZE ; i++ )
	table[i] = NULL;
}


hash(char *s)
{
    int		i, hval;

    for( i=NPRIMES-1, hval=0 ; i>=0 && *s != '\0' ; i--, s++ )
	hval += primes[i]*(*s);
    return(hval % TABLESIZE);
}


/* Insert term into chained hash table. */
node *
insert(char *term)
{
    node	*tmp;
    int		hval = hash(term);

    tmp = (node *) malloc(sizeof(node));
    tmp->term = (char *) malloc(strlen(term) + 1);
    strcpy(tmp->term, term);
    tmp->next = table[hval];
    table[hval] = tmp;

    return(tmp);
}


/* Find term in a hash table. */
node *
find(char *term)
{
    int		hval = hash(term);
    node	*curr;

    for( curr=table[hval] ; curr != NULL && strcmp(term, curr->term) != 0 ;
			    curr = curr->next )
	;
    return(curr);
}
